#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
WebSocket客户端示例 - 用于测试语音识别服务
"""

import asyncio
import json
import websockets
import argparse
import numpy as np
import wave
import os


def parse_args():
    """解析命令行参数"""
    parser = argparse.ArgumentParser(description="FunASR WebSocket客户端")
    
    parser.add_argument(
        "--host", 
        type=str, 
        default="localhost", 
        help="服务器主机地址"
    )
    
    parser.add_argument(
        "--port", 
        type=int, 
        default=10095, 
        help="服务器端口"
    )
    
    parser.add_argument(
        "--audio_file", 
        type=str, 
        required=True, 
        help="要识别的音频文件路径"
    )
    
    parser.add_argument(
        "--mode", 
        type=str, 
        default="2pass",
        choices=["2pass", "online", "offline"],
        help="识别模式: 2pass(默认), online, offline"
    )
    
    parser.add_argument(
        "--chunk_size", 
        type=str, 
        default="5,10", 
        help="分块大小, 格式为'encoder_size,decoder_size'"
    )
    
    parser.add_argument(
        "--use_ssl", 
        action="store_true", 
        help="是否使用SSL连接"
    )
    
    return parser.parse_args()


async def send_audio(websocket, audio_file, mode, chunk_size):
    """
    发送音频文件到服务器进行识别
    
    参数:
        websocket: WebSocket连接
        audio_file: 音频文件路径
        mode: 识别模式
        chunk_size: 分块大小
    """
    # 打开并读取WAV文件
    with wave.open(audio_file, "rb") as wav_file:
        params = wav_file.getparams()
        frames = wav_file.readframes(wav_file.getnframes())
    
    # 音频文件信息
    print(f"音频文件: {os.path.basename(audio_file)}")
    print(f"采样率: {params.framerate}Hz, 通道数: {params.nchannels}")
    print(f"采样位深: {params.sampwidth * 8}位, 总帧数: {params.nframes}")
    
    # 设置配置参数
    config = {
        "mode": mode,
        "chunk_size": chunk_size,
        "wav_name": os.path.basename(audio_file),
        "is_speaking": True
    }
    
    # 发送配置
    await websocket.send(json.dumps(config))
    
    # 模拟实时发送音频数据
    chunk_size_bytes = 3200  # 每次发送100ms的16kHz音频
    total_chunks = len(frames) // chunk_size_bytes
    
    print(f"开始发送音频数据，共 {total_chunks} 个数据块...")
    
    try:
        for i in range(0, len(frames), chunk_size_bytes):
            chunk = frames[i:i+chunk_size_bytes]
            await websocket.send(chunk)
            
            # 模拟实时，每100ms发送一次
            await asyncio.sleep(0.1)
            
            # 显示进度
            if (i // chunk_size_bytes) % 10 == 0:
                print(f"已发送 {i // chunk_size_bytes}/{total_chunks} 数据块")
        
        # 发送结束信号
        await websocket.send(json.dumps({"is_speaking": False}))
        print("音频数据发送完成")
        
    except Exception as e:
        print(f"发送音频时出错: {e}")


async def receive_results(websocket):
    """
    接收并显示识别结果
    
    参数:
        websocket: WebSocket连接
    """
    online_text = ""
    offline_text = ""
    
    try:
        async for message in websocket:
            # 解析服务器返回的JSON消息
            result = json.loads(message)
            
            mode = result.get("mode", "")
            text = result.get("text", "")
            is_final = result.get("is_final", False)
            
            # 根据模式更新文本
            if "online" in mode:
                online_text = text
                print(f"\r[在线识别] {online_text}", end="", flush=True)
            elif "offline" in mode:
                offline_text = text
                print(f"\n[离线识别] {offline_text}")
            
            # 如果是最终结果，打印完整信息
            if is_final and offline_text:
                print("\n最终识别结果:")
                print(f"[离线识别] {offline_text}")
                return
                
    except Exception as e:
        print(f"接收结果时出错: {e}")


async def main():
    """主函数"""
    args = parse_args()
    
    # WebSocket URI
    protocol = "wss" if args.use_ssl else "ws"
    uri = f"{protocol}://{args.host}:{args.port}"
    
    print(f"连接到服务器: {uri}")
    
    try:
        # 创建WebSocket连接
        async with websockets.connect(
            uri, 
            subprotocols=["binary"]
        ) as websocket:
            
            print("连接成功")
            
            # 创建两个任务: 发送音频和接收结果
            send_task = asyncio.create_task(
                send_audio(websocket, args.audio_file, args.mode, args.chunk_size)
            )
            
            receive_task = asyncio.create_task(
                receive_results(websocket)
            )
            
            # 等待任务完成
            await asyncio.gather(send_task, receive_task)
            
    except Exception as e:
        print(f"连接服务器失败: {e}")


if __name__ == "__main__":
    # 运行主函数
    asyncio.run(main()) 